#include <QDebug>
#include <QFileDialog>
#include <QDesktopWidget>
#include <QSplitter>
#include <QScrollArea>
#include <QMessageBox>
#include <QTabWidget>
#include <QSettings>
#include "Q7BFormDesigner.h"
#include "ui_Q7BFormDesigner.h"
#include "wpalette.h"
#include "propertyeditor.h"
#include "objecttreewindow.h"
#include "formdesignersession.h"
#include "widgetfactory.h"
#include "basicmethodtranslator.h"

Q7BFormDesigner *GLOBAL_Q7BFormDesigner = 0;



void Q7BFormDesigner::emitSignalToIDE_AddSignal(QString sCode)
{
    emit addSignal(sFileName, sCode);
}

Q7BFormDesigner::Q7BFormDesigner (QWidget *parent) :
    QMainWindow (parent),
    ui (new Ui::Q7BFormDesigner)
{
    GLOBAL_Q7BFormDesigner = this;

    setGeometry ( 0  , 0 , 1024 , 768 ) ;
    move ( QApplication::desktop ()->screen ()->width () / 2 - width () / 2 ,
          QApplication::desktop ()->screen ()->height () / 2 - height () / 2 ) ;

    //this->menuBar()->hide();

    ui->setupUi (this);
    QSplitter * mainSplitter = new QSplitter ( Qt::Horizontal , this ) ;
    mainSplitter->setOpaqueResize ( false ) ;
    centralWidget ()->layout ()->addWidget ( mainSplitter ) ;

    QWidget *ww = new QWidget();
    QWidget *www = new QWidget();
    QVBoxLayout *ll = new QVBoxLayout();
    ww->setLayout(ll);
    ll->setSpacing(0);
    ll->setContentsMargins(0, 0, 0, 0);
    QHBoxLayout *lll = new QHBoxLayout();

   // QToolButton *File = new QToolButton( ); File->setText("File"); File->setMenu(new QMenu()); File->setPopupMode(QToolButton::InstantPopup); File->setMinimumWidth(100); lll->addWidget(File);
    QToolButton *Edit = new QToolButton( ); Edit->setText("Edit"); Edit->setMenu(new QMenu()); Edit->setPopupMode(QToolButton::InstantPopup); Edit->setMinimumWidth(100); lll->addWidget(Edit);
    QToolButton *View = new QToolButton( ); View->setText("View"); View->setMenu(new QMenu()); View->setPopupMode(QToolButton::InstantPopup); View->setMinimumWidth(100); lll->addWidget(View);
    QToolButton *Insert = new QToolButton( ); Insert->setText("Insert"); Insert->setMenu(new QMenu()); Insert->setPopupMode(QToolButton::InstantPopup); Insert->setMinimumWidth(100); lll->addWidget(Insert);

    www->setLayout(lll);
    lll->setSpacing(0);
    lll->setContentsMargins(0, 0, 0, 0);

    ll->addWidget(www);

    palettes = new QTabWidget( this ) ;
    ll->addWidget(palettes);
    palettes->setTabPosition( QTabWidget::West ) ;
    mainSplitter->addWidget ( ww ) ;//palettes

    foreach( QString cat , WidgetFactory::instance()->getCategories() )
    {
        wp = new WidgetsPalette ( this , cat ) ;
        wp->show () ;
        QScrollArea * paletteScrollArea = new QScrollArea( this ) ;
        paletteScrollArea->setFrameShape(QFrame::NoFrame); // BN
        palettes->addTab( paletteScrollArea , cat ) ;
        paletteScrollArea->setWidget( wp  ) ;
        paletteScrollArea->setMinimumWidth( wp->width() ) ;
        paletteScrollArea->setHorizontalScrollBarPolicy ( Qt::ScrollBarAlwaysOff ) ;
    }

    mdiArea = new QMdiArea  ( this ) ;
    mdiArea->setHorizontalScrollBarPolicy ( Qt::ScrollBarAsNeeded ) ;
    mdiArea->setVerticalScrollBarPolicy ( Qt::ScrollBarAsNeeded ) ;
    mdiArea->setLineWidth ( 2 ) ;
    mdiArea->setFrameShape ( QFrame::StyledPanel )  ;
    mainSplitter->addWidget ( mdiArea ) ;

    newSessionAction = new QAction ( "New form" , this ) ;
    deleteAction = new QAction ( "&Delete Item" , this) ;
    selectAllAction = new QAction ( "Select all" , this ) ;
    loadMaketAction = new QAction ( "Load form" , this ) ;
    saveMaketAction = new QAction ( "Save form" , this ) ;
    alignLeftAction = new QAction ( "Align left" , this ) ;
    alignTopAction = new QAction ( "Align top" , this ) ;
    alignRightAction = new QAction ( "Align right" , this ) ;
    alignBottomAction = new QAction ( "Align bottom" , this ) ;

    moveLeftAction = new QAction ( "Move left" , this ) ;
    moveTopAction = new QAction ( "Move top" , this ) ;
    moveRightAction = new QAction ( "Move right" , this ) ;
    moveBottomAction = new QAction ( "Move bottom" , this ) ;

    copyAction = new QAction ( "&Copy" , this ) ;
    pasteAction = new QAction ( "&Paste" , this ) ;
    cutAction = new QAction ( "C&ut" , this ) ;
    undoAction = new QAction ( "Undo" , this ) ;
    redoAction = new QAction ( "Redo" , this ) ;
    gridSwitchAction = new QAction ( "Grid" , this ) ;
    showCursorPosSwitchAction = new QAction ( "Show cursor pos" , this ) ;

    moveToBackAction = new QAction ( "Move to back" , this ) ;
    moveToFrontAction = new QAction ( "Move to front" , this ) ;

    deleteAction->setShortcut ( Qt::Key_Delete ) ;
    copyAction->setShortcut ( QKeySequence ( Qt::CTRL + Qt::Key_C ) ) ;
    pasteAction->setShortcut ( QKeySequence ( Qt::CTRL + Qt::Key_V ) ) ;
    cutAction->setShortcut ( QKeySequence ( Qt::CTRL + Qt::Key_X ) ) ;
    undoAction->setShortcuts (QKeySequence::Undo);
    redoAction->setShortcuts (QKeySequence::Redo);

    switchHeaderAction = new QAction ( "Header enabled" , this ) ;
    switchHeaderAction->setCheckable ( true ) ;
    switchHeaderAction->setChecked ( true ) ;

    switchFooterAction = new QAction ( "Footer enabled" , this ) ;
    switchFooterAction->setCheckable ( true ) ;
    switchFooterAction->setChecked ( true ) ;

    autoSectionsSizeAction = new QAction ( "Sections autoresizing" , this ) ;
    autoSectionsSizeAction->setCheckable ( true ) ;
    autoSectionsSizeAction->setChecked ( true ) ;

    setFormSizeAction = new QAction ( "Set form size" , this ) ;
    connect ( setFormSizeAction , SIGNAL (triggered ()) , SLOT (setFormSizeSlot ())) ;
    gridSwitchAction->setCheckable ( true ) ;
    gridSwitchAction->setChecked ( true ) ;

    showCursorPosSwitchAction->setCheckable ( true ) ;
    showCursorPosSwitchAction->setChecked ( true ) ;

    copyPropertiesAction = new QAction ( "Copy properties to buffer" , this ) ;
    assignPropertiesAction = new QAction ( "Assign properties from buffer" , this ) ;

    editTabOrderAction = new QAction ( "Edit tab order" , this ) ;

    makeRelGeomAction = new QAction ( "Make geometry related" , this ) ;
    makeUnRelGeomAction = new QAction ( "Make geometry non-related" , this ) ;

    switchShowWidgetNameAction = new QAction ( "Show widget name" , this ) ;
    switchShowWidgetNameAction->setCheckable ( true ) ;
    switchShowWidgetNameAction->setChecked ( true ) ;

    setSameHeightAction = new QAction ( "Set same height" , this ) ;
    setSameWidthAction = new QAction ( "Set same width" , this ) ;
    setSameSizeAction = new QAction ( "Set same size" , this ) ;

    togglePaletteAction = new QAction ( "Show palette" , this ) ;
    togglePaletteAction->setCheckable ( true ) ;
    togglePaletteAction->setChecked ( true ) ;
    connect ( togglePaletteAction , SIGNAL(triggered()) , SLOT(togglePalette()) ) ;
    togglePropEditorAction = new QAction ( "Show editor" , this ) ;
    togglePropEditorAction->setCheckable ( true ) ;
    togglePropEditorAction->setChecked ( true ) ;
    connect ( togglePropEditorAction , SIGNAL(triggered()) , SLOT(togglePropEditor()) ) ;
    toggleTreeAction = new QAction ( "Show tree" , this ) ;
    toggleTreeAction->setCheckable ( true ) ;
    toggleTreeAction->setChecked ( true ) ;
    connect ( toggleTreeAction , SIGNAL(triggered()) , SLOT(toggleTree()) ) ;

    showPreviewAction = new QAction ( "Show preview" , this ) ;

    setNoLayoutAction = new QAction ( "Remove layout" , this ) ;
    setVertLayoutAction = new QAction ( "Set vertical layout" , this) ;
    setHorLayoutAction = new QAction ( "Set horizontal layout" , this) ;

#ifndef Q7BASIC // activate for testing purpose only
    fileMenu = menuBar ()->addMenu ( tr ( "&File" ) ) ;
    fileMenu->addAction ( newSessionAction ) ;
    fileMenu->addAction ( loadMaketAction ) ;
    fileMenu->addAction ( saveMaketAction ) ;
#endif

    //editMenu = menuBar ()->addMenu (tr ("&Edit"));
    Edit->menu()->addAction ( undoAction ) ;
    Edit->menu()->addAction (redoAction ) ;
    Edit->menu()->addSeparator () ;
    Edit->menu()->addAction ( deleteAction ) ;
    Edit->menu()->addSeparator () ;
    Edit->menu()->addAction ( copyAction ) ;
    Edit->menu()->addAction ( pasteAction ) ;
    Edit->menu()->addAction ( cutAction ) ;
    Edit->menu()->addSeparator () ;
    Edit->menu()->addAction ( selectAllAction ) ;
    Edit->menu()->addSeparator () ;
    Edit->menu()->addAction ( switchHeaderAction ) ;
    Edit->menu()->addAction ( switchFooterAction ) ;
    Edit->menu()->addSeparator () ;
    QMenu * morphMenu = Edit->menu()->addMenu( "Morph into..." ) ;
    QStringList morphNames = WidgetFactory::instance()->getClassNames() ;
    foreach ( QString cn , morphNames )
    {
        QAction * n = new QAction( cn , this ) ;
        connect( n , SIGNAL(triggered()) , SLOT(morphSlot()) ) ;
        morphMenu->addAction( n ) ;
        morphActions.append( n ) ;
    }
    Edit->menu()->addSeparator () ;

    moveMenu = Edit->menu()->addMenu ( tr  ( "Move" ) ) ;
    moveMenu->addAction ( moveLeftAction ) ;
    moveMenu->addAction ( moveTopAction ) ;
    moveMenu->addAction ( moveRightAction ) ;
    moveMenu->addAction ( moveBottomAction ) ;
    Edit->menu()->addSeparator () ;

    alignMenu = Edit->menu()->addMenu ( tr  ( "Align" ) ) ;
    alignMenu->addAction ( alignLeftAction ) ;
    alignMenu->addAction ( alignTopAction ) ;
    alignMenu->addAction ( alignRightAction ) ;
    alignMenu->addAction ( alignBottomAction ) ;
    Edit->menu()->addSeparator () ;
    Edit->menu()->addAction ( setFormSizeAction ) ;
    Edit->menu()->addSeparator () ;
    Edit->menu()->addAction ( autoSectionsSizeAction ) ;
    Edit->menu()->addSeparator () ;
    Edit->menu()->addAction ( copyPropertiesAction ) ;
    Edit->menu()->addAction ( assignPropertiesAction ) ;

    Edit->menu()->addSeparator () ;
    Edit->menu()->addAction ( editTabOrderAction ) ;

    Edit->menu()->addSeparator () ;
    Edit->menu()->addAction ( makeRelGeomAction ) ;
    Edit->menu()->addAction ( makeUnRelGeomAction ) ;
    Edit->menu()->addSeparator () ;

    QMenu * zoOrderMenu = Edit->menu()->addMenu( "Z-order..." ) ;
    zoOrderMenu->addAction( moveToBackAction ) ;
    zoOrderMenu->addAction( moveToFrontAction ) ;
    Edit->menu()->addSeparator () ;
    Edit->menu()->addAction ( setSameHeightAction ) ;
    Edit->menu()->addAction ( setSameWidthAction ) ;
    Edit->menu()->addAction ( setSameSizeAction ) ;

    QMenu * layoutMenu = Edit->menu()->addMenu ( "Layouts..." ) ;
    layoutMenu->addAction( setNoLayoutAction ) ;
    layoutMenu->addAction( setVertLayoutAction ) ;
    layoutMenu->addAction( setHorLayoutAction ) ;

    Edit->menu()->addSeparator () ;

    //viewMenu = menuBar ()->addMenu (tr ("&View")) ;
    View->menu()->addAction ( gridSwitchAction ) ;
    View->menu()->addAction ( showCursorPosSwitchAction ) ;
    View->menu()->addAction ( switchShowWidgetNameAction ) ;
    View->menu()->addSeparator () ;
    View->menu()->addAction( togglePaletteAction ) ;
    View->menu()->addAction( togglePropEditorAction ) ;
    View->menu()->addAction( toggleTreeAction ) ;
    View->menu()->addSeparator () ;
    View->menu()->addAction ( showPreviewAction ) ;

   // insertMenu = menuBar()->addMenu( "Insert" ) ;
    insertMenu = Insert->menu();
    foreach ( QString cat , WidgetFactory::instance()->getCategories() )
    {
        QMenu * m = insertMenu->addMenu ( cat ) ;
        QStringList l = WidgetFactory::instance()->getClassNamesByCat( cat );
        l.sort();

        QChar c0;
        QChar c1;

        foreach ( QString name , l )
        {
            if (name.at(0) != QChar('Q') && name.at(0) != c0) m->addSeparator();
            if (name.at(0) == QChar('Q') && name.at(1) != c1) m->addSeparator();
            c0 = name.at(0);
            c1 = name.at(1);

            QAction * a = m->addAction( name ) ;
            connect ( a , SIGNAL(triggered()) , SLOT(insertWidgetActionSlot()) ) ;
        }
    }

    insertMenu->addSeparator();

    QStringList l = WidgetFactory::instance()->getClassNames();
    l.sort();
    QChar c0;
    QChar c1;
    foreach ( QString name , l )
    {
        if (name.at(0) != QChar('Q') && name.at(0) != c0) insertMenu->addSeparator();
        if (name.at(0) == QChar('Q') && name.at(1) != c1) insertMenu->addSeparator();
        c0 = name.at(0);
        c1 = name.at(1);
        QAction * a = insertMenu->addAction( name ) ;
        connect ( a , SIGNAL(triggered()) , SLOT(insertWidgetActionSlot()) ) ;
    }

    PE = new PropertyEditor ( this ) ;
    OW = new ObjectTreeWindow ( this ) ;

    QSplitter * splitter = new QSplitter ( Qt::Vertical , this ) ;
    splitter->setOpaqueResize ( false ) ;
    mainSplitter->addWidget ( splitter ) ;
    splitter->addWidget ( OW ) ;
    splitter->addWidget ( PE ) ;

    connect ( newSessionAction , SIGNAL (triggered ()) , SLOT (createSession ())) ;

    connect ( WidgetFactory::instance() , SIGNAL(widgetsChanged()) , OW , SLOT(scanMaket()) , Qt::QueuedConnection ) ;
    mouseInsertMode = false ;
    loadAppSettings () ;

//#ifndef Q7BASIC // activate for testing purpose only
    createSession () ;
//#endif

}

Q7BFormDesigner::~Q7BFormDesigner ()
{
    saveAppSettings () ;
    delete ui;
}

void Q7BFormDesigner::loadForm(QString fileName)
{   
  sFileName = fileName;

  //createSession();
  if (!QFile::exists(fileName)){ // if not exist, create a new one and save it

    saveForm(fileName);
  } else {
    session->loadForm(fileName);
  }
  setWindowTitle(QFileInfo(fileName).baseName() + ".Form.q7b");
}

void Q7BFormDesigner::saveForm(QString fileName)
{
  sFileName = fileName;

  session->saveForm(fileName);
}

void Q7BFormDesigner::createSession ()
{
    session = new FormDesignerSession ( mdiArea , this ) ;
    mdiArea->tileSubWindows ();
    connect ( mdiArea , SIGNAL (subWindowActivated (QMdiSubWindow*)) ,
             session , SLOT (windowActivated (QMdiSubWindow*))) ;

    sessions.append ( session ) ;
    connect (deleteAction, SIGNAL (triggered ()), session, SLOT (deleteItem ( )));
    connect (copyAction, SIGNAL (triggered ()),  session,SLOT (copyItem ()));
    connect (pasteAction, SIGNAL (triggered ()),  session, SLOT ( pasteToRoot ()));
    connect (cutAction, SIGNAL (triggered ()),  session, SLOT (cutItem ()));
    connect (gridSwitchAction, SIGNAL (triggered (bool)),  session, SLOT (switchGrid (bool)));
    connect (showCursorPosSwitchAction, SIGNAL (triggered (bool)),  session, SLOT (switchShowCursorPos (bool)));
    connect (switchShowWidgetNameAction, SIGNAL (triggered (bool)),  session, SLOT (switchShowWidgetName (bool)));


    connect ( selectAllAction , SIGNAL (triggered ()) ,  session, SLOT (selectAll ()) ) ;
    connect ( loadMaketAction , SIGNAL (triggered ()) , session, SLOT (loadForm ()) ) ;
    connect ( saveMaketAction , SIGNAL (triggered ()) , session, SLOT (saveForm ()) ) ;

    connect ( alignLeftAction , SIGNAL (triggered ()) , session, SLOT (alignLeft ()) ) ;
    connect ( alignTopAction , SIGNAL (triggered ()) ,  session,SLOT (alignTop ()) ) ;
    connect ( alignRightAction , SIGNAL (triggered ()) , session, SLOT (alignRight ()) ) ;
    connect ( alignBottomAction , SIGNAL (triggered ()) , session, SLOT (alignBottom ()) ) ;

    connect ( moveLeftAction , SIGNAL (triggered ()) , session, SLOT ( moveLeft ()) ) ;
    connect ( moveTopAction , SIGNAL (triggered ()) ,  session,SLOT ( moveTop ()) ) ;
    connect ( moveRightAction , SIGNAL (triggered ()) , session, SLOT ( moveRight () ) ) ;
    connect ( moveBottomAction , SIGNAL (triggered ()) , session, SLOT ( moveBottom () ) ) ;

    connect ( copyPropertiesAction , SIGNAL (triggered ()) , session, SLOT (copyProperties ()) ) ;
    connect ( assignPropertiesAction , SIGNAL (triggered ()) , session, SLOT (assignProperties ()) ) ;
    connect ( switchHeaderAction , SIGNAL (triggered (bool)) , session , SLOT (switchHeader (bool)) ) ;
    connect ( switchFooterAction , SIGNAL (triggered (bool)) , session , SLOT (switchFooter (bool)) ) ;

    connect ( autoSectionsSizeAction , SIGNAL (triggered (bool)) , session , SLOT (setMaketSectionsAutoResize (bool)) ) ;

    connect ( undoAction , SIGNAL (triggered ()) , session, SLOT (undo ()) ) ;
    connect ( redoAction , SIGNAL (triggered ()) , session, SLOT (redo ()) ) ;

    connect ( editTabOrderAction , SIGNAL ( triggered() ) , session , SLOT ( startTabOrderEditing () ) ) ;

    connect ( makeRelGeomAction , SIGNAL ( triggered() ) , session , SLOT ( makeGeometryRelated() ) ) ;
    connect ( makeUnRelGeomAction , SIGNAL ( triggered() ) , session , SLOT ( makeGeometryUnRelated() ) ) ;

    connect ( PE , SIGNAL (propsEdited (QVariantMap)) , session,SLOT (propsEdited (QVariantMap)) ) ;

    connect ( session , SIGNAL (wmChanged (WidgetManager*)) , PE , SLOT (wmChanged (WidgetManager*))) ;
    connect ( session , SIGNAL (wmChanged (WidgetManager*)) , OW , SLOT (wmChanged (WidgetManager*))) ;

    connect ( session , SIGNAL (selectionChanged ()) , PE , SLOT (selectionChanged ()) ) ;
    connect ( session , SIGNAL (maketChanged (FormMaket*)) , OW , SLOT (setMaket (FormMaket*))) ;
    connect ( session , SIGNAL (structureChanged ()) , OW , SLOT (scanMaket ()) ) ;
    connect ( session , SIGNAL (selectionChanged ()) , OW , SLOT (selectionChanged ())) ;
    connect ( OW , SIGNAL (selectWidgets (QList<QPointer<QWidget> >,bool)) ,
             session , SLOT (selectItems (QList<QPointer<QWidget> >,bool))) ;

    connect ( session , SIGNAL (contextMenuSignal ( QWidget * )) , SLOT (widgetContextMenu ( QWidget *))) ;
    connect ( session , SIGNAL (activated (FormDesignerSession*)) , SLOT (sessionActivated (FormDesignerSession*))) ;

    connect ( moveToBackAction , SIGNAL(triggered()) , session , SLOT( moveToBack() ) ) ;
    connect ( moveToFrontAction , SIGNAL(triggered()) , session , SLOT( moveToFront() ) ) ;

    connect ( setSameHeightAction , SIGNAL(triggered()) , session,SLOT(setSameHeight())) ;
    connect ( setSameWidthAction , SIGNAL(triggered()) , session,SLOT(setSameWidth())) ;
    connect ( setSameSizeAction , SIGNAL(triggered()) , session,SLOT(setSameSize())) ;

    connect ( showPreviewAction , SIGNAL(triggered()) , session , SLOT(showPreview()) ) ;

    connect(  session , SIGNAL(leftClicked()) , SLOT(maketLeftClicked())) ;
    connect ( this , SIGNAL(mouseInsert()) , session , SLOT(pasteToRoot())) ;

    connect( setNoLayoutAction , SIGNAL(triggered()) , session , SLOT( setNoLayout()) ) ;
    connect( setHorLayoutAction , SIGNAL(triggered()) , session , SLOT( setHorLayout()) ) ;
    connect( setVertLayoutAction , SIGNAL(triggered()) , session , SLOT( setVertLayout()) ) ;

    session->windowActivated ( mdiArea->currentSubWindow () ) ;
    sessionActivated ( session ) ;
}

void Q7BFormDesigner::widgetContextMenu ( QWidget * w )
{
    if ( w )
    {
        QMenu menu (this);

        QList < QPointer < QAction > > customActList = WidgetFactory::instance()->getSpecialActions(w);
        if (! customActList.isEmpty())
        {
            foreach ( QPointer < QAction > pAction , customActList )
                menu.addAction( pAction ) ;
            menu.addSeparator() ;
        }

        menu.addAction ( deleteAction ) ;
        menu.addAction ( copyAction ) ;
        menu.addAction ( cutAction ) ;
        menu.addSeparator() ;

        menu.addAction ( copyPropertiesAction ) ;
        menu.addAction ( assignPropertiesAction ) ;
        menu.addSeparator() ;

        QMenu * mm = menu.addMenu( "Morph into..." ) ;
        foreach( QAction * a , morphActions )
            mm->addAction( a ) ;
        menu.addSeparator() ;
        menu.addAction ( makeRelGeomAction ) ;
        menu.addAction ( makeUnRelGeomAction ) ;
        menu.addSeparator() ;
        QMenu * sm = menu.addMenu( "Signals..." ) ;

        signalsActions.clear() ;
        foreach( BasicMethodTranslator bm , BasicMethodTranslator::fromQObject ( w ) )
        {
            if (bm.isSignal())
            {
                signalsActions.append( sm->addAction( bm.basicLangVariant() ) ) ;
                connect( signalsActions.last().data() , SIGNAL(triggered()) , SLOT(signalSelected()) ) ;
            }
        }
        menu.addSeparator() ;
        menu.addAction( moveToBackAction ) ;
        menu.addAction( moveToFrontAction ) ;
        menu.addSeparator() ;
        QMenu * layoutMenu = menu.addMenu ( "Layouts..." ) ;
        layoutMenu->addAction( setNoLayoutAction ) ;
        layoutMenu->addAction( setVertLayoutAction ) ;
        layoutMenu->addAction( setHorLayoutAction ) ;

        menu.exec ( QCursor::pos () ) ;
    }
    else
    {
        QMenu menu (this);
        menu.addAction ( pasteAction ) ;
        menu.exec ( QCursor::pos () ) ;
    }
}



void Q7BFormDesigner::sessionActivated ( FormDesignerSession * session )
{
    //foreach ( FormDesignerSession * s , sessions )
    //    s->setActive ( false ) ;

    if ( ! session )
        return ;
    activeSession = session ;
    //activeSession->setActive ( true ) ;
    gridSwitchAction->setChecked ( activeSession->gridState () ) ;
    autoSectionsSizeAction->setChecked ( activeSession->isMaketSectionsAutoresize () ) ;
    switchShowWidgetNameAction->setChecked( activeSession->isShowWidgetName() ) ;
    showCursorPosSwitchAction->setChecked( activeSession->getShowCursorPos() ) ;
}

void Q7BFormDesigner::setFormSizeSlot ()
{
    SizeDialog sd ;
    sd.setSize ( activeSession->getMdiWindow ()->size () ) ;
    sd.exec () ;
    if ( activeSession )
        activeSession->getMdiWindow ()->setGeometry ( QRect ( activeSession->getMdiWindow ()->geometry ().topLeft () ,
                                                       sd.getSize () ) ) ;
}

void Q7BFormDesigner::morphSlot()
{
    QAction * a = qobject_cast < QAction * > ( sender () ) ;
    if( a )
        activeSession->morphWidgets( a->text() ) ;
}

void Q7BFormDesigner::signalSelected()
{
    QAction * a = qobject_cast < QAction * > ( sender() ) ;
    if ( a )
    {
        QWidget * w = activeSession->getSelectedWidgets().first() ;
        if ( w )
        {
            QString signalSignature = a->text() ;
#ifndef Q7BASIC // activate for testing purpose only
            QMessageBox msgBox ;
            msgBox.setText ( QString( "on_%1_%2" ).arg( w->objectName() ).arg( signalSignature ) ) ;
            msgBox.exec () ;
#endif
            GLOBAL_Q7BFormDesigner->emitSignalToIDE_AddSignal(QString( "on_%1_%2" ).arg( w->objectName() ).arg( signalSignature ));
        }
    }
}

void Q7BFormDesigner::togglePalette ( )
{
    palettes->setVisible ( togglePaletteAction->isChecked() ) ;
}

void Q7BFormDesigner::togglePropEditor (  )
{
    PE->setVisible ( togglePropEditorAction->isChecked() ) ;
}

void Q7BFormDesigner::toggleTree (  )
{
    OW->setVisible ( toggleTreeAction->isChecked() ) ;
}

void Q7BFormDesigner::saveAppSettings ()
{
    QSettings s ( "settings.ini" , QSettings::IniFormat ) ;
    s.setValue( "showPalette" , togglePaletteAction->isChecked() ) ;
    s.setValue( "showEditor" , togglePropEditorAction->isChecked() ) ;
    s.setValue( "showTree" , toggleTreeAction->isChecked() ) ;
    s.setValue( "Q7BFormDesignerGeometry" , geometry() ) ;
}

void Q7BFormDesigner::loadAppSettings ()
{
    QSettings s ( "settings.ini" , QSettings::IniFormat ) ;
    togglePaletteAction->setChecked ( s.value ( "showPalette" , true ).toBool () ) ;
    togglePalette() ;
    togglePropEditorAction->setChecked ( s.value ( "showPalette" , true ).toBool () ) ;
    togglePropEditor () ;
    toggleTreeAction->setChecked ( s.value ( "showTree" , true ).toBool () ) ;
    toggleTree () ;
    setGeometry( s.value( "Q7BFormDesignerGeometry" , QRect( 10 , 10 , 1000 , 550 ) ).toRect() ) ;
}

void Q7BFormDesigner::maketLeftClicked ()
{
    if ( mouseInsertMode )
    {
        mouseInsertMode = false ;
        emit mouseInsert() ;
    }
}

void Q7BFormDesigner::insertWidgetActionSlot ()
{
    QAction * a = qobject_cast < QAction * > ( sender() ) ;
    QString name = a->text() ;
    QWidget * w = WidgetFactory::instance()->newWidgetByClassName ( name , "" , true ) ;
    if ( w )
    {
        WidgetFactory::instance()->copyToCB( w ) ;
        mouseInsertMode = true ;
        QMessageBox msgBox ;
        msgBox.setText ( "Click on the position where to insert the widget." ) ;
        msgBox.exec () ;
    }
}


SizeDialog::SizeDialog (  ) : QDialog ()
{
    layout = new QFormLayout ( this ) ;
    setLayout ( layout ) ;
    he = new QSpinBox ( this ) ;
    he->setMaximum ( 10000 ) ;
    wi = new QSpinBox ( this ) ;
    wi->setMaximum ( 10000 ) ;

    layout->addRow ( "Width:" , wi ) ;
    layout->addRow ( "Height:" , he ) ;
    setWindowTitle ( "Set window size" ) ;
}
void SizeDialog::setSize ( QSize s )
{
    wi->setValue ( s.width () ) ;
    he->setValue ( s.height () ) ;
}

QSize SizeDialog::getSize ()
{
    QSize s ;
    s.setWidth ( wi->value () ) ;
    s.setHeight ( he->value () ) ;
    return s ;
}

void SizeDialog::keyPressEvent (QKeyEvent * e)
{
    qDebug () << e->key () ;
    if ( e->key () == Qt::Key_Return )
        done ( 0 ) ;
}


