#include <QDebug>
#include <QHBoxLayout>
#include "formmaket.h"
#include "designingarea.h"
#include "qresizeblesplitter.h"
#include "relativelayout.h"
#include "constants.h"
#include <QResizeEvent>
#include <QDateTime>
#include "widgetfactory.h"
FormMaket::FormMaket (  QWidget *parent) :
    QWidget (parent)
  , isInit ( false )
  , gridEnabled ( true )
  , policy ( AUTO )
{
    animationGroup = new QParallelAnimationGroup ( this ) ;
}

void FormMaket::init ()
{
    if ( ! isInit )
    {
        createAreas () ;
        connectAreas () ;
        isInit = true ;
    }
}

void FormMaket::connectAreas ()
{

    foreach ( DesigningArea * ar , areas )
    {
        connect ( ar , SIGNAL (movingWidgets (QPoint)) , this , SIGNAL (movingWidgets (QPoint))) ;
        connect ( ar , SIGNAL (resizeWidgets (QPoint,int)) , this , SIGNAL (resizeWidgets (QPoint,int))) ;
        connect ( ar , SIGNAL (endMoving ()) , this , SIGNAL (endMoving ())) ;
        connect ( ar , SIGNAL (endResizing ()) , this , SIGNAL (endResizing ())) ;
        connect ( ar , SIGNAL (inFocus (DesigningArea *)) , SLOT (focusChanged (DesigningArea *))) ;
        connect ( ar , SIGNAL (moveWidgetByPaste ( QWidget * , QPoint)) ,
                 SIGNAL (moveWidgetByPaste ( QWidget *, QPoint))) ;
        connect ( ar , SIGNAL (contextMenuSignal ( QWidget * )) , SIGNAL (contextMenuSignal ( QWidget *))) ;
        connect ( ar , SIGNAL (widgetsDragged (QWidget *)) ,
                 SIGNAL (widgetsDragged (QWidget *))) ;
        connect ( ar , SIGNAL (widgetsSelected (QList<QPointer<QWidget> >,bool)) , SIGNAL (widgetsSelected (QList<QPointer<QWidget> >,bool))) ;
        connect ( ar , SIGNAL (clearSelection ()) , SIGNAL (clearSelection ())) ;
        connect ( ar , SIGNAL(widgetTooltip(QWidget*)) , SIGNAL(widgetTooltip(QWidget*))) ;
        connect( ar , SIGNAL(leftClicked()) , SIGNAL(leftClicked())) ;
    }
}

void FormMaket::addArea ( DesigningArea * da )
{
    if ( ! areas.contains ( da ) )
        areas.append ( da ) ;
}

void FormMaket::focusChanged (DesigningArea * da )
{
    foreach ( DesigningArea * ar , areas )
    {
        ar->drawFocus ( false ) ;
        if ( ar == da )
            da->drawFocus ( true ) ;
    }
    emit inFocus ( da ) ;
}

void FormMaket::setGrid ( bool b )
{
    if ( b )
        foreach ( DesigningArea * ar , areas )
            ar->setGrid () ;
    else
        foreach ( DesigningArea * ar , areas )
            ar->setGrid ( 1 ) ;
    gridEnabled = b ;
}

void FormMaket::setShowCursorPos ( bool b )
{
    if ( b )
        foreach ( DesigningArea * ar , areas )
            ar->setShowCursorPos ( true ) ;
    else
        foreach ( DesigningArea * ar , areas )
            ar->setShowCursorPos ( false ) ;
}

bool FormMaket::getShowCursorPos ( )
{
    if ( ! areas.isEmpty() )
        return areas.first()->getShowCursorPos() ;
    return false ;
}

void FormMaket::enlargeGeom ( int w , int h )
{
    DesigningArea * area = qobject_cast< DesigningArea * >  ( sender () ) ;
    if ( area )
    {
        animationGroup->clear () ;
        QPropertyAnimation * a = new QPropertyAnimation ( area , "geometry" , this ) ;
        a->setDuration ( 200 ) ;
        a->setEasingCurve ( QEasingCurve::InOutQuint ) ;
        a->setStartValue ( area->geometry () ) ;
        a->setEndValue ( QRect ( area->geometry ().topLeft () , QSize ( w , h ) ) ) ;
        animationGroup->addAnimation ( a ) ;

        foreach ( DesigningArea * ar , areas )
        {
            if ( ar != area )
            {
                QPropertyAnimation * a = new QPropertyAnimation ( ar , "geometry" , this ) ;
                a->setDuration ( 200 ) ;
                a->setEasingCurve ( QEasingCurve::InOutQuint ) ;
                a->setStartValue ( ar->geometry () ) ;
                a->setEndValue ( QRect ( ar->geometry ().topLeft () , QSize ( w , ar->height () ) ) ) ;
                animationGroup->addAnimation ( a ) ;
            }
        }
        animationGroup->start ();
    }
}

QList < DesigningArea * > FormMaket::getAreas ()
{
    return areas ;
}

int FormMaket::getGridStep ()
{
    if ( ! areas.isEmpty() )
        return areas.first()->getGridDelta() ;
    return - 1 ;
}


DesigningArea * FormMaket::sectionByName ( QString n )
{
    foreach ( DesigningArea * ar , areas )
    {
        if ( ar->objectName () == n )
            return ar ;
    }
    return 0 ;
}

QString FormMaket::nameOfSection ( DesigningArea * n )
{
    return n->objectName () ;
}

void FormMaket::setAreaVisible ( QString name , bool b )
{
    foreach ( DesigningArea * ar , areas )
    {
        if ( ar->objectName () == name )
            ar->setVisible ( b ) ;
    }
}

bool FormMaket::isAreaVisible ( QString name )
{
    foreach ( DesigningArea * ar , areas )
    {
        if ( ar->objectName () == name )
            return ar->isVisible () ;
    }
    return false ;
}

bool FormMaket::gridState (  )
{
    return gridEnabled ;
}

void FormMaket::updateSectionSize ( DesigningArea * a )
{
    if ( areas.contains ( a ) )
        a->setGeometry ( geometry () ) ;
}

void FormMaket::setSectionsSizePolicy ( SectionsSizePolicy p )
{
    policy = p ;
    if ( policy == AUTO )
        updateSectionSize ( 0 ) ;
}

bool FormMaket::isSectionsAutoResizeble ()
{
    return policy == AUTO ;
}
void FormMaket::setMode ( bool b )
{
    foreach ( DesigningArea * ar , areas )
        ar->setMode ( b ) ;
}

void FormMaket::closeEvent ( QCloseEvent *)
{
    emit closing ();
}

QVariantMap FormMaket::splitterPositions()
{
    return  QVariantMap () ;
}

void FormMaket::setSplitterPositions ( QVariantMap )
{

}


ThreeSectionsMaket::ThreeSectionsMaket ( QWidget *parent) :
    FormMaket ( parent )
{
    setLayout ( new QHBoxLayout ( this ) ) ;
    spl = new QResizebleSplitter ( this ) ;
    spl->setOpaqueResize ( false ) ;
    connect ( spl , SIGNAL (splitterMoved (int,int)) , SLOT (splitterMoved (int,int))) ;
    setMouseTracking ( true ) ;
    layout ()->addWidget ( spl ) ;
    setGeometry ( QRect ( 0 , 0 , 640 , 480 ) ) ;
}

void ThreeSectionsMaket::createAreas ()
{
    headerScrollArea = new QScrollArea ( spl ) ;
    daHeader = new DesigningArea ( headerScrollArea ) ;
    daHeader->setLayout( new RelativeLayout ( daHeader ) ) ;
    daHeader->setObjectName ( "Header" ) ;
    connect ( daHeader , SIGNAL (geomSmall (int,int)) , SLOT (enlargeGeom (int,int)) ) ;
    daHeader->setGeometry ( 0 , 0 , 400 , 100 ) ;
    daHeader->show () ;
    addArea ( daHeader ) ;
    headerScrollArea->setWidget ( daHeader ) ;

    bodyScrollArea = new QScrollArea ( spl ) ;
    daBody = new DesigningArea ( bodyScrollArea ) ;
    daBody->setLayout( new RelativeLayout ( daBody ) ) ;
    daBody->setObjectName ( "Body" ) ;
    connect ( daBody , SIGNAL (geomSmall (int,int)) , SLOT (enlargeGeom (int,int)) ) ;

    daBody->setGeometry ( 0 , 0 , 400 , 300 ) ;
    daBody->show () ;
    addArea ( daBody ) ;
    bodyScrollArea->setWidget ( daBody ) ;

    footerScrollArea = new QScrollArea ( spl ) ;
    daFooter = new DesigningArea ( footerScrollArea ) ;
    daFooter->setLayout( new RelativeLayout ( daFooter ) ) ;
    daFooter->setObjectName ( "Footer" ) ;
    connect ( daFooter , SIGNAL (geomSmall (int,int)) , SLOT (enlargeGeom (int,int)) ) ;

    daFooter->setGeometry ( 0 , 0 , 400 , 100 ) ;
    daFooter->show () ;
    addArea ( daFooter ) ;
    footerScrollArea->setWidget ( daFooter ) ;

    spl->addWidget ( headerScrollArea ) ;
    spl->addWidget ( bodyScrollArea ) ;
    spl->addWidget ( footerScrollArea ) ;
}

void ThreeSectionsMaket::setAreaVisible ( QString name , bool b )
{
    if ( name == daHeader->objectName () )
        headerScrollArea->setVisible ( b ) ;
    else if ( name == daFooter->objectName () )
        footerScrollArea->setVisible ( b ) ;
    updateSectionSize ( 0 ) ;
}

void ThreeSectionsMaket::splitterMoved ( int  , int  )
{
    updateSectionSize ( 0 ) ;
}

void ThreeSectionsMaket::resizeEvent (QResizeEvent * )
{
    updateSectionSize ( 0 ) ;
}

void ThreeSectionsMaket::updateSectionSize ( DesigningArea * )
{
    int minWidth = 0 ;
    foreach ( DesigningArea * ca , getAreas() )
    {
        QRect commonEdge ;
        ca->setMinimumWidth ( 0 ) ;
        ca->setMinimumHeight ( 0 ) ;
        foreach ( QObject * s , ca->children () )
        {
            if ( s->property ( SEL_SQUARES_MARK ).toBool () == true )
                continue ;
            QWidget * w = qobject_cast< QWidget * > ( s ) ;
            if ( w && ! w->property( DELETED_PROP_NAME ).toBool()
                 && WidgetFactory::instance()->isClassSupported(w->metaObject()->className()))
            {
                commonEdge = commonEdge.united ( w->geometry () ) ;
                //qDebug( ) << QDateTime::currentDateTime().toString() << ca->objectName() << w->objectName() ;
            }
        }
        if ( minWidth < commonEdge.right () )
            minWidth = commonEdge.right () ;

        if ( commonEdge.right () > 0 )
            ca->setMinimumWidth ( commonEdge.right () ) ;
        if ( commonEdge.bottom () > 0 )
            ca->setMinimumHeight ( commonEdge.bottom () ) ;
    }
    //qDebug() << QDateTime::currentDateTime().toString() << "updated" ;
    foreach ( DesigningArea * ca , getAreas() )
        ca->setMinimumWidth ( minWidth ) ;

    if ( policy == AUTO )
    {
        int sizeDecr = 2 ;
        daHeader->setGeometry ( QRect ( daHeader->geometry().topLeft()
                                        , headerScrollArea->geometry ().size () - QSize ( sizeDecr , sizeDecr ) ) ) ;
        daBody->setGeometry ( QRect ( daBody->geometry().topLeft()
                                      , bodyScrollArea->geometry ().size () - QSize ( sizeDecr , sizeDecr ) ) ) ;
        daFooter->setGeometry ( QRect ( daFooter->geometry().topLeft()
                                        , footerScrollArea->geometry ().size () - QSize ( sizeDecr , sizeDecr ) ) ) ;
    }
}


QVariantMap ThreeSectionsMaket::splitterPositions ()
{
    return spl->positions() ;
}

void ThreeSectionsMaket::setSplitterPositions ( QVariantMap p )
{
    spl->setPositions ( p ) ;
}
