#include "objecttreewindow.h"
#include "widgetfactory.h"
#include "designingarea.h"
#include "formmaket.h"
#include "widgetmanager.h"
#include "constants.h"
#include <QHBoxLayout>
#include <QDebug>
#include "props_functions.h"
ObjectTreeWindow::ObjectTreeWindow ( QWidget *parent) :
    QWidget (parent) , WM ( 0 )
{
    tree = new QTreeWidget ( this ) ;
    tree->setSelectionMode ( QAbstractItemView::ExtendedSelection ) ;
    connect ( tree , SIGNAL (itemClicked (QTreeWidgetItem*,int)) , SLOT (itemClicked (QTreeWidgetItem*,int)) ) ;
    connect( tree , SIGNAL(itemSelectionChanged()) , SLOT(itemSelectionChanged())) ;
    QHBoxLayout *l = new QHBoxLayout  ( this ); // BN
    l->setSpacing(0); // BN
    l->setContentsMargins(0, 0, 0, 0); // BN

    setLayout  ( l ) ;
    layout ()->addWidget ( tree ) ;
    tree->setColumnCount ( 1 ) ;
    tree->setHeaderLabels ( QStringList () << "Widget name" ) ;
    tree->show () ;
}

void ObjectTreeWindow::scanMaket ( )
{
    tree->clear () ;
    hash.clear () ;
    itemsByName.clear () ;
    if ( ! m )
        return ;
    QTreeWidgetItem * rootItem = new QTreeWidgetItem ( tree ) ;
    rootItem->setText ( 0 , "Form" ) ;
    foreach ( QWidget * area , m->getAreas () )
    {
        if ( area->isVisible () )
        {
            QTreeWidgetItem * areaItem = new QTreeWidgetItem ( rootItem ) ;
            areaItem->setText ( 0 , area->objectName () ) ;
            itemsByName.insert ( area->objectName () , areaItem ) ;
            hash.insert ( areaItem , area ) ;

            foreach ( QWidget * child , WidgetFactory::instance ()->getChildren ( area ) )
            {

                if ( child && ! child->property( DELETED_PROP_NAME ).toBool () )
                    scan ( areaItem , child ) ;
            }
        }
    }
    //tree->sortByColumn ( 0 ,Qt::AscendingOrder ) ;
    tree->expandAll () ;
    selectionChanged () ;
}

void ObjectTreeWindow::scan ( QTreeWidgetItem * item , QWidget * w )
{
    QTreeWidgetItem * childItem = new QTreeWidgetItem ( item ) ;

    childItem->setText ( 0 , w->objectName () + " As " + w->metaObject()->className() ) ;
    itemsByName.insert ( w->objectName () , childItem ) ;
    hash.insert ( childItem , w ) ;

    foreach ( QWidget * child , WidgetFactory::instance ()->getChildren ( w ) )
    {
        if ( child && ! child->property( DELETED_PROP_NAME ).toBool () )
            scan ( childItem , child ) ;
    }
}

void ObjectTreeWindow::setMaket ( FormMaket * maket )
{
    m = maket ;
    scanMaket () ;
}

void ObjectTreeWindow::itemClicked  ( QTreeWidgetItem * , int )
{
    QList < QPointer < QWidget > > list ;
    foreach ( QTreeWidgetItem * item , tree->selectedItems  () )
        list << hash.value ( item ) ;
    emit selectWidgets (  list  , false ) ;
}

void ObjectTreeWindow::selectionChanged ()
{
    if ( WM )
    {
        tree->clearSelection () ;

        if ( ! WM->getCurrentSelList ().isEmpty () )
        {
            foreach ( QWidget * w , WM->getCurrentSelList () )
            {
                if ( w )
                {
                    QString name = w->objectName () ;
                    if ( itemsByName.contains ( name ) )
                        itemsByName.value ( name )->setSelected ( true ) ;
                }
            }
        }
    }
}

void ObjectTreeWindow::wmChanged ( WidgetManager * w )
{
    WM = w ;
    if ( WM )
        selectionChanged (  ) ;
}

void ObjectTreeWindow::itemSelectionChanged ()
{
    bool flag = false ;
    foreach ( QTreeWidgetItem * item , tree->selectedItems  () )
        if( item->parent() && item->parent()->parent () == 0 )
        {
            if( ! flag )
                flag = true ;
            else
                item->setSelected ( false ) ;
        }
}
