#include "relativelayout.h"
#include "constants.h"
#include "widgetfactory.h"
#include <QVariant>
#include <QWidget>
#include <QDebug>
RelativeLayout::RelativeLayout ( QWidget * parent ) :
    QLayout( parent )
{
}

RelativeLayout::~RelativeLayout ()
{
    QLayoutItem * item ;
    while ( ( item = takeAt ( 0 ) ) )
        delete item ;
}

void RelativeLayout::addItem ( QLayoutItem * item )
{
    list.append ( item ) ;
}

QSize RelativeLayout::sizeHint () const
{
    QSize s( 0 , 0 ) ;
    int n = list.count () ;
    if ( n > 0 )
        s = QSize ( 10 , 10 ) ; //start with a nice default size
    int i = 0 ;
    while ( i < n )
    {
        QLayoutItem * o = list.at ( i ) ;
        s = s.expandedTo ( o->sizeHint ( ) ) ;
        ++ i ;
    }
    return s + n * QSize ( spacing () , spacing () ) ;
}

QSize RelativeLayout::minimumSize () const
{
    return QSize ( 0 , 0 ) ;
}

int RelativeLayout::count () const
{
    return list.size () ;
}

QLayoutItem * RelativeLayout::itemAt ( int idx ) const
{
    return list.value ( idx ) ;
}

QLayoutItem * RelativeLayout::takeAt ( int idx )
{
    return idx >= 0 && idx < list.size () ? list.takeAt ( idx ) : 0 ;
}

void RelativeLayout::setGeometry ( const QRect & r )
{
    QLayout::setGeometry ( r ) ;
    //if ( prevGeom == r )
    //    return ;
    prevGeom = r ;
    if ( list.size () == 0 )
        return ;

    int i = 0;
    while ( i < list.size() )
    {
        QLayoutItem * o = list.at ( i ) ;
        QWidget * w = o->widget () ;
        if ( w )
        {
            int lS = w->property( REL_L ).toInt() ;
            int rS = w->property( REL_R ).toInt() ;
            int tS = w->property( REL_T ).toInt() ;
            int bS = w->property( REL_B ).toInt() ;

            int nX = w->geometry().x() ;
            int nY = w->geometry().y() ;
            int nW = w->geometry().width() ;
            int nH = w->geometry().height() ;

            //qDebug () << w->objectName() << w->geometry () ;
            QRect parentGeom = w->parentWidget()->geometry() ;

            QList < QWidget * > siblList ;

//            foreach ( QObject * ch , w->parent()->children() )
//            {
//                QWidget * siblingW = qobject_cast < QWidget * > ( ch ) ;
//                if ( siblingW
//                     && WidgetFactory::instance()->isClassSupported( siblingW->metaObject()->className() )
//                     && siblingW != w
//                     && ! siblingW->property( DELETED_PROP_NAME ).toBool())
//                    siblList.append ( siblingW ) ;
//            }

            QRect rectWithMargins = w->geometry () ;
            if( w->property( EN_REL_L ).toBool() )
                rectWithMargins.adjust( - lS , 0 , 0 , 0 ) ;
            if( w->property( EN_REL_T ).toBool() )
                rectWithMargins.adjust( 0 , - tS , 0 , 0 ) ;
            if( w->property( EN_REL_R ).toBool() )
                rectWithMargins.adjust( 0 , 0 , rS , 0 ) ;
            if( w->property( EN_REL_B ).toBool() )
                rectWithMargins.adjust( 0 , 0 , 0 , bS ) ;


            QRect leftZone = QRect ( rectWithMargins.left()
                                     , rectWithMargins.top()
                                     , lS
                                     , rectWithMargins.height()) ;
            QRect topZone = QRect ( rectWithMargins.left() + lS
                                    , rectWithMargins.top()
                                    , rectWithMargins.width() - lS - rS
                                    , rectWithMargins.height() / 2 ) ;
            QRect rightZone = QRect ( rectWithMargins.right() - rS
                                      , rectWithMargins.top()
                                      , rS
                                      , rectWithMargins.height()) ;
            QRect bottomZone = QRect ( rectWithMargins.left() + lS
                                       , rectWithMargins.center().y()
                                       , rectWithMargins.width() - lS - rS
                                       , rectWithMargins.height() / 2 ) ;

            // may be implemented through QRect::intersected!
            bool needToUpd = true ;
            while ( needToUpd )
            {
                int maxLeft = 0 ;
                int minRight = parentGeom.width() ;
                int maxTop = 0 ;
                int minBottom = parentGeom.height() ;

                needToUpd = false ;
                //LEFT SIDE
                if( w->property( EN_REL_L ).toBool() )
                {
                    maxLeft += lS ;
                    QWidget * closest = 0 ;
                    if ( ! siblList.isEmpty() )
                    {
                        closest = siblList.first() ;
                        foreach ( QWidget * siblingW , siblList )
                            if ( siblingW->geometry().right() > closest->geometry().right() )
                                closest = siblingW ;
                    }
                    if ( closest && closest->property( EN_REL_R ).toBool() )
                    {
                        double koef = double(closest->property( REL_R ).toInt()) / double(lS);
                        int sp = qMax(closest->property( REL_R ).toInt(),lS);
                        nX = closest->geometry().left()
                                + koef * ( w->geometry().right() - closest->geometry().left()) + sp / 2;

                    }
                    else
                    {
                        if ( closest && closest->geometry().left() <= w->geometry().right()
                             && closest->geometry().intersects ( leftZone )
                             && closest->geometry().center().x() <= w->geometry().center().x() )
                        {
                            if ( closest->geometry().right() + lS > maxLeft )
                                maxLeft = closest->geometry().right() + lS - 2 ;
                        }

                        nX = maxLeft ;
                    }
                }
                //TOP SIDE
                if( w->property( EN_REL_T ).toBool() )
                {
                    maxTop += tS ;

                    QWidget * closest = 0 ;
                    if ( ! siblList.isEmpty() )
                    {
                        closest = siblList.first() ;
                        foreach ( QWidget * siblingW , siblList )
                            if ( siblingW->geometry().bottom() > closest->geometry().bottom() )
                                closest = siblingW ;
                    }

                    if ( closest && closest->geometry().top() <= w->geometry().bottom()
                         && closest->geometry().intersects ( topZone )
                         && closest->geometry().center().y() <= w->geometry().center().y() )
                    {
                        if ( closest->geometry().bottom() + tS > maxTop )
                            maxTop = closest->geometry().bottom() + tS - 2 ;
                    }

                    nY = maxTop ;
                }
                //RIGHT SIDE
                if( w->property( EN_REL_R ).toBool() )
                {
                    minRight -= rS - 1 ; // :)

                    QWidget * closest = 0 ;
                    if ( ! siblList.isEmpty() )
                    {
                        closest = siblList.first() ;
                        foreach ( QWidget * siblingW , siblList )
                            if ( siblingW->geometry().left() < closest->geometry().left() )
                                closest = siblingW ;
                    }

                    if ( closest && closest->geometry().right() >= w->geometry().left()
                         && closest->geometry().intersects ( rightZone )
                         && closest->geometry().center().x() > w->geometry().center().x() )
                    {
                        if ( closest->geometry().left() - rS < minRight )
                            minRight = closest->geometry().left() - rS + 2 ;
                    }

                    nW = minRight - nX ;
                }
                //BOTTOM SIDE
                if( w->property( EN_REL_B ).toBool() )
                {
                    minBottom -= bS - 1 ; // :)

                    QWidget * closest = 0 ;
                    if ( ! siblList.isEmpty() )
                    {
                        closest = siblList.first() ;
                        foreach ( QWidget * siblingW , siblList )
                            if ( siblingW->geometry().top() < closest->geometry().top() )
                                closest = siblingW ;
                    }

                    if ( closest && closest->geometry().bottom() >= w->geometry().top()
                         && closest->geometry().intersects ( bottomZone )
                         && closest->geometry().center().y() > w->geometry().center().y() )
                    {
                        if ( closest->geometry().top() - bS < minBottom )
                            minBottom = closest->geometry().top() - bS + 2 ;
                    }

                    nH = minBottom - nY ;
                }

                if ( nW < w->minimumWidth() ) nW = w->minimumWidth() ;
                if ( nH < w->minimumHeight() ) nH = w->minimumHeight() ;

                //                if ( w->geometry() != QRect( nX , nY , nW , nH ) )
                //                    needToUpd = true ;

                w->setGeometry( QRect( nX , nY , nW , nH ) ) ;
            }
            //qDebug( ) << w->geometry() ;
            emit widgetGeomUpdated ( w ) ;
        }
        ++ i ;
    }
}
